
( function( $, srf ) {


    // Strict mode eliminates some JavaScript pitfalls
    'use strict';

    // Ensure the namespace is initialized and available
    srf.formats = srf.formats || {};

    /**
     * Base constructor for objects representing a radar instance
     * @since 1.8
     * @type Object
     */

    srf.formats.radar = function() {};

    srf.formats.radar.prototype = {
        // Specification of functions and parameters
        show: function( context ) {
            return context.each( function() {
                var w = $( this ).width(),
                    h = $( this ).height(),
                    chart  = $( this ).find( '.container' ),
                    radarID   = chart.attr( 'id' ),
                    json   = mw.config.get( radarID );
                       
                util.spinner.hide( { context: $( this ) } );

                var container = typeof json === 'string' ? jQuery.parseJSON( json ) : json;
                var legend  = container.parameters.legend,
                    middle   = container.parameters.middle;
        

                //Fetch data
                var initial = container.data;
                
                //Arange data to an array of objects like this:
                /*
                    [
                        [
                            {"axis":"sector1","name":data1,"type":0,"value":1},
                            {"axis":"sector2","name":data2,"type":1,"value":2},
                            {"axis":"sector3","name":data3,"type":2,"value":3},
                            {"axis":"sector4","name":data4,"type":3,"value":4}
                        ],

                        [
                            {"axis":"sector1","name":data5,"type":0,"value":5},
                            {"axis":"sector2","name":data6,"type":1,"value":4},
                            {"axis":"sector3","name":data7,"type":2,"value":3}
                        ],
                        
                        [
                            {"axis":"sector1","name":data8,"type":0,"value":2}
                        ]
                    ];
                */

                //Sort by value.length descending
                initial.sort(function (a, b) {
                    return b.value.length - a.value.length;
                });

                var array1 = [];
                var array2 = [];
        
                //Check which label hast more than 2 values 
                for (var i = 0; i < initial.length; i++){
                    
                    if(initial[i].value.length > 2 && initial[i].value.length % 2 == 0 ){
                        
                        for (var j = 0; j < initial[i].value.length; j++) {

                            array1.push( initial[i].axis[0] , initial[i].value[j] );
                            
                        }
                    }
                    else for (var j = 0; j < initial[i].value.length; j++) {
                        
                            array2.push( initial[i].axis[0] , initial[i].value[j] );
                    }        
                }

                //Delete unnecessary label
                for (var k = 2; k < array1.length;  k = k+3) {
                        
                        array1.splice(k,1);
                        array1.slice(0,3);
                }
                
                for (var k = 2; k < array2.length;  k = k+3) {

                        array2.splice(k,1);
                }
        
                //Function to group entries in one array with labels and values
                function divide(a, n) {
                    var len = a.length,out = [], i = 0;
                    while (i < len) {
                        var size = Math.ceil((len - i) / n--);
                        out.push(a.slice(i, i += size));
                    }
                    return out;
                }
                
                var array3 = divide(array1,array1.length/3); 
                var array4 = divide(array2,array2.length/3);

                //Function to convert arrays to objects
                function toObject(arr){
                    var data = [];
                    for (var m = 0; m < arr.length; ++m ) {
                        var obj = {};
                        
                        obj.axis = arr[m][0];
                        obj.value = arr[m][1][1];
                        obj.type = arr[m][2][1];
                        obj.name = arr[m][1][0];
                     
                    data.push(obj);        
                    
                    }
                    return data;
                }; 
    
                var obj1 = toObject(array3);
                var obj2 = toObject(array4);
                var compare = [];
                for (var i = 0; i < initial[0].value.length/2-1; i++){
                    compare.push([i]);    
                };
        
                //Dummy object for not getting out of bounds when deleting
                var dummyobj = {};
                    dummyobj.axis = "DUMMYOBJECT";
                    dummyobj.value = "0";
                    dummyobj.type = "0";
                
                obj1.push(dummyobj);

                //Count different axis names where value is > 2
                for (var i = 0; i < initial.length; i++) {
                    if (initial[i].value.length > 2) {
                        var count = i+1;
                    } 
                };
        
                //Compare the name of the properties, delete and distribute into new arrays    
                for (var j = 0; j < count; j++ ) {   
                    while (obj1[j].axis === obj1[j+1].axis){
                        for (var i = 0; i < initial[j].value.length/2-1; i++){
                            compare[i].push(obj1[j+1])
                            obj1.splice(j+1,1);
                        }
                    }        
                };
       
                //Pop the dummyobject
                obj1.pop();
        
                //Delete the first empty indeces 
                for (var i = 0; i < compare.length; i++) {
                    compare[i].shift();
                }; 

                //Create aranged data object
                var mainarray = obj1.concat(obj2);
                var d = [mainarray];
                for (var i = 0; i < compare.length; i++) {
                    d.push(compare[i]);
                };
                for (var i = 0; i < d.length; i++){
                    for (var j = 0; j < d[i].length; j++){
                        if (d[i][j].type > 3){
                            d[i][j].type = "circle";
                        }    
                        else if (d[i][j].type === 1){
                            d[i][j].type = "triangle-down";
                        }
                        else if (d[i][j].type === 2){
                            d[i][j].type = "square";
                        }
                        else if (d[i][j].type === 3){
                            d[i][j].type = "diamond";
                        }
                        else if (d[i][j].type <= 0) {
                            d[i][j].type = "circle";
                        }
                    }
                };

                var RadarChart = {
                    draw: function (id, d, options) {
                        //Settings
                        var cfg = {
                            w: w,
                            h: h,
                            factor: 1,
                            factorLegend: 0.85,
                            levels: 5,
                            maxValue: 12,
                            radians: Math.PI*2 ,
                            opacityArea: 0.2,
                            ToRight: 5,
                            TranslateX: 100,
                            TranslateY: 0,
                            ExtraWidthX: 300,
                            ExtraWidthY: 50,
                        };

                    cfg.maxValue = Math.max(cfg.maxValue, d3.max(d, function (i) {
                        return d3.max(i.map(function (o) {
                            return o.value;
                        }))
                    }));
                    var allAxis = (d[0].map(function (i, j) {
                        return i.axis;
                    }));
                    var total = allAxis.length;
                    var radius = cfg.factor * Math.min(cfg.w / 2, cfg.h / 2);
                    var Format = d3.format('');
                    d3.select(id).select("svg").remove();
                    var g = d3.select(id)
                        .append("svg")
                        .attr("width", cfg.w + cfg.ExtraWidthX)
                        .attr("height", cfg.h + cfg.ExtraWidthY)
                        .append("g")
                        .attr("transform", "translate(" + cfg.TranslateX + "," + cfg.TranslateY + ")");;
    
                    //Create circles within radar lines
                    var circle1 = g.append("circle")
                        .attr("cx", cfg.w/2)
                        .attr("cy", cfg.h/2)
                        .attr("r", cfg.factor*6.5*(cfg.w/14)/3)
                        .style("fill","grey")
                        .style("opacity",0.5)
                        .style("stroke", "black");        

                    var circle2 = g.append("circle")
                        .attr("cx", cfg.w/2)
                        .attr("cy", cfg.h/2)
                        .attr("r", cfg.factor*6.5*((cfg.w/14)/1.5))
                        .style("fill","lightgrey")
                        .style("opacity",0.5)
                        .style("stroke", "black");

                    var circle3 = g.append("circle")
                        .attr("cx", cfg.w/2)
                        .attr("cy", cfg.h/2)
                        .attr("r", cfg.factor*6.5*cfg.w/14)
                        .style("fill","white")
                        .style("opacity",0.5)
                        .style("stroke", "black");         

                    //Create axis 
                    var axis = g.selectAll("axis")
                        .data(allAxis)
                        .enter()
                        .append("g")
                        .attr("class", "axis");

                    axis.append("line")
                        .attr("x1", cfg.w / 2 )
                        .attr("y1", cfg.h / 2 )
                        .attr("x2", function (d, i) {
                        return cfg.w / 2 * (1 - cfg.factor * Math.sin((i+1) * cfg.radians / total));
                    })
                        .attr("y2", function (d, i) {
                        return cfg.h / 2 * (1 - cfg.factor * Math.cos((i+1) * cfg.radians / total));
                    })
                        .attr("class", "line")
                        .style("stroke", "grey")
                        .style("stroke-width", "1px");

                    axis.append("text")
                        .attr("class", "legend")
                        .text(function (d) {
                        return d;
                    })
                        .style("font-family", "sans-serif")
                        .style("font-size", "15px")
                        .attr("text-anchor", "middle")
                        .attr("dy", "1.2em")
                        .attr("transform", function (d, i) {
                        return "translate(1, -19)"
                    })
                        .attr("x", function (d, i) {
                        return cfg.w / 2 * (1 -  Math.sin((i + 0.5) * cfg.radians / total)) - 30 * Math.sin((i+0.5) * cfg.radians / total);
                    })
                        .attr("y", function (d, i) {
                        return cfg.h / 2 * (1 - Math.cos((i + 0.5) * cfg.radians / total)) - 1 * Math.cos((i+0.5) * cfg.radians / total);
                    });

                    var colors = {"diamond": "red", "square": "orange", "triangle-down": "steelblue", "circle": "black"};
                    
                    //D3.svg Symbols
                    d.forEach(function (z) {
                        
                        g.selectAll("symbols")
                            .data(z)
                            .enter()
                            .append("path")
                            .attr("class", "symbols")
                            .text(function (j) { return j.name })
                            .attr("transform", function(j,i){return "translate("+cfg.w / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.sin((i+0.5) * cfg.radians / total))+","+cfg.h / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.cos((i+0.5) * cfg.radians / total))+")"})
                            .attr("d", d3.svg.symbol().type(function(d) { return d.type; }).size( 90))
                            .style("fill", "white")
                            .style("stroke", function(d){ return colors[d.type];})
                            .style("stroke-width", "1.5px")      
                            .on('mouseover', function (d) {                  
                                tooltip.transition(200)
                                .style('opacity', 1);
                            })
                            .on('mouseout', function () {
                                tooltip.transition(200)
                                .style('opacity', 0);
                            g.selectAll("polygon")
                                .transition(200)
                                .style("fill-opacity", cfg.opacityArea);
                        })
                            .append("svg:title")
                            .text(function (j) { return j.value });
                    });

                    //Tooltip
                    var tooltip = g.append('text')
                        .style('opacity', 0)
                        .style('font-family', 'sans-serif')
                        .style('font-size', '13px');

                    //Show label on the right
                    function showLegendRight(x) {
                        d.forEach(function (y) {

                            g.selectAll("label")
                                .data(y)
                                .enter()
                                .append("text") 
                                .text(function (j) { return j.name })
                                .attr("transform", function(j,i){return "translate("+cfg.w / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.sin((i+0.5) * cfg.radians / total))+","+cfg.h / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.cos((i+0.5) * cfg.radians / total))+")rotate("+x+")"})
                                .attr("x", 9)
                                .attr("id", "label"+x+"")
                                .attr("dy", ".35em") 
                                .attr("fill", "black")
                                .attr("text-anchor", "front")
                                .attr("font-size", "9.5px")       
                        });
                    };

                    //Show label on the left
                    function showLegendLeft(x) {
                        d.forEach(function (y) {

                            g.selectAll("label")
                                .data(y)
                                .enter()
                                .append("text") 
                                .text(function (j) { return j.name })
                                .attr("transform", function(j,i){return "translate("+cfg.w / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.sin((i+0.5) * cfg.radians / total))+","+cfg.h / 2 * (1 - (j.value / cfg.maxValue ) * cfg.factor * Math.cos((i+0.5) * cfg.radians / total))+")rotate("+x+")"})
                                .attr("x", -9)
                                .attr("id", "label"+x+"")
                                .attr("dy", ".35em") 
                                .attr("fill", "black")
                                .attr("text-anchor", "end")
                                .attr("font-size", "9.5px")         
                        });
                    };

                    //Values for label rotation
                    var x = [0,10, 20, 30, 40, 50, 60, 70, 80, 90, -90, -80, -70, -60, -50, -40, -30, -20, -10, 0];
                    var y = 0;

                    //Rotate label clockwise
                    function rotateRight() {
                        d3.selectAll("#label"+x[y-1]+"").remove();
                        showLegendRight(x[y]);
                        y = y + 1;
                        if (y > x.length) {window.location.reload();};
                    };

                    //Rotate label anti-clockwise
                    function rotateLeft() {
                        d3.selectAll("#label"+x[y-1]+"").remove();
                        showLegendLeft(x[y]);
                        y = y + 1;
                        if (y > x.length) {window.location.reload();};
                    };

                    //Fetch Html button by ID and rotate clockwise or anti-clockwise on click
                    document.getElementById('rotateRight').addEventListener('click', rotateRight);
                    document.getElementById('rotateLeft').addEventListener('click', rotateLeft);

                    //Function to reset labels
                    function resetLegend() {
                        RadarChart.draw("#" + radarID, d);
                    };

                    //Reset labels on click
                    window.onload = function() {
                    var resetButton =  document.getElementById("reset");
                        resetButton.onclick = resetLegend;      
                    };
        
                    //Accessor function 
                    var lineFunction = d3.svg.line()
                        .x(function (d) { return d.x; })
                        .y(function (d) { return d.y; })
                        .interpolate("linear");

                    //Set legend options
                    var LegendOptions2 = [ "triangle-down",  "square",  "diamond", "circle"];
                    var LegendOptions3 = [ 'Niedrig', 'Mittel', 'Hoch', 'Keine Angabe'];
                    var colors = {"triangle-down": "steelblue", "square": "orange", "diamond": "red", "circle": "black"};

                    var svg = d3.select('#' + radarID)
                        .selectAll('svg')
                        .append('svg')
                        .attr("width", w+300)
                        .attr("height", h)

                    //Create the title for the legend
                    var text = svg.append("text") 
                        .attr("x", w+142)
                        .attr("y", h/40)
                        .attr("text-anchor", "front")
                        .attr("font-size", "12px")
                        .attr("fill", "#404040")
                        .text(""+legend+":");

                    var text = svg.append("text")
                        .attr("x", w+181)
                        .attr("y", h-h/8)
                        .attr("dy", "-0.25em") 
                        .attr("text-anchor", "front")
                        .attr("font-size", "12px")
                        .attr("fill", "#404040")
                        .text(""+middle+"");
    
                    //Initiate Legend for symbols
                    var legend2 = svg.append("g")
                        .attr("class", "legend")    

                    //Text for axis ticks
                    legend2.selectAll('path')
                        .data(LegendOptions2)
                        .enter()
                        .append("path")
                        .attr("class", "symbols")
                        .attr("transform", function(d,i){ return "translate("+(w+150)+","+(i * 20 + h/12)+")";})
                        .attr("d", d3.svg.symbol().type(function(d) {return d;}))
                        .style("fill", "white")
                        .attr("text-anchor", "front")
                        .attr("font-size", "11px")
                        .attr ("stroke", function(d){ return colors[d];})
                        .text(function (d) {
                        return d;
                        });
            
                    var legend3 = svg.append("g")
                        .attr("class", "legend")

                    legend3.selectAll('text')
                        .data(LegendOptions3)
                        .enter()
                        .append("text")
                        .attr("x", w + 170  )
                        .attr("y", function (d, i) {
                            return i * 20 + h/12;
                        }) 
                        .attr("dy", ".35em")  
                        .attr("text-anchor", "front")
                        .attr("font-size", "11px")
                        .attr("fill", "#737373")
                        .text(function (d) { return d; }); 

                    //Legend for the meaning of the middle of the radar
                    var circleLegend1 = g.append("circle")
                        .attr("cx", w+51)
                        .attr("cy", h-h/8)
                        .attr("r", 30)
                        .style("fill","white")
                        .style("opacity",0.5)
                        .style("stroke", "black");        

                    var circleLegend2 = g.append("circle")
                        .attr("cx", w+51)
                        .attr("cy", h-h/8)
                        .attr("r", 10)
                        .style("fill","grey")
                        .style("opacity",0.5)
                        .style("stroke", "black");

                    var circleLegend3 = g.append("circle")
                        .attr("cx", w+51)
                        .attr("cy", h-h/8)
                        .attr("r", 20)
                        .style("fill","lightgrey")
                        .style("opacity",0.5)
                        .style("stroke", "black");

                    var circleLegend4 = g.append("circle")
                        .attr("cx", w+51)
                        .attr("cy", h-h/8)
                        .attr("r", 2)
                        .style("fill","black")
                        .style("opacity",0.5)
                        .style("stroke", "black");    

                    var arrowline = g.append("line")
                        .attr("x1", w+70)
                        .attr("y1", h-h/8)
                        .attr("x2", w+135)
                        .attr("y2", h-h/8)
                        .attr("stroke-width", 2)
                        .attr("stroke", "black");

                    var arrowhead = g.append("path")
                        .attr("d","M "+(w+55)+" "+(h-h/8)+" L "+(w+70)+" "+(h-h/7.2)+" L "+(w+70)+" "+(h-h/9)+" L "+(w+55)+" "+(h-h/8)+"")
                        .style("stroke-width", 1)
                        .style("stroke", "black")
                        .style("fill", "black");  
                }
            }
        
        RadarChart.draw("#" + radarID, d);
        
        })
    }
}

    /**
     * Implementation and representation of the radar instance
     * @since 1.8
     * @type Object
     */

    // Create class instance
    var radar = new srf.formats.radar();

    // Get access to SRF specific utilities function
    var util = new srf.util();

    $( document ).ready(function() {

        // Use the class selector to find all instances relevant to the "radar" printer
        // since a wiki page can have more than one instance of the same result printer
        // .each() ensures instances are handled separately
        $( '.srf-radar' ).each(function() {

            // Access methods available through the radar class
            radar.show( $( this ) );
        
        } );
    } );
} )( jQuery, semanticFormats );